## forecasting exercise
nsim <- 10000

## source main file (for globals and all)
if (!exists("set_globals")) {
  print("Has export_results.R been sourced before?")
  stop(0)
}
out_timing <- "Tue Dec 21 18.01.23 2021"  ## all results; correct, but AIG in 0.1bp
# out_timing <- "Wed Dec 22 09.49.41 2021"  ## 31-61 results; correct, AIG in bp
stress_scenarios <- list(
  c(0,0,-10,  -10,0,0,  0,0,0,  0,0,0),
  NULL
)
## set main choices for models
models <- c(3,4)
optionset <- c(1)
subsidy_sets <- list( 
  s1 = c('S')
  # s1 = c('SubsIP', 'TarpIP', 'FedSurplusIP','AigIP')
)
## lag macro (as covariate) by a month (1) or do not (0) lag as covariate ... 
## macro as dependent is times the same as rating experience
macro_lag <- 1

## set folder to fetch data
datadirectory <- "../../data/new/"
## rating file name
rating_filename <- "transitions_2scale_RatingDate.csv"
## set start and end date of sample to be used
start_date <- as.Date("1998-01-01")
end_date <- as.Date("2021-09-30")
## set start of covid sample
covid_start_date <- as.Date("2020-01-01")
## output file name for all estimates across models, samples, controls
timing_name <- stri_replace_all(date(), ".", regex = "[:;/-]")
Total_results_name <- paste0("Total-results ", timing_name, ".csv")
Output_directory_name <- paste0("./forecast ", timing_name, "/")
if (!dir.exists(Output_directory_name)) dir.create(Output_directory_name)

## Set globals
samplechoice <- "precov"
for (subsidy in subsidy_sets) for (stress_scenario in stress_scenarios) for (mymodel in models) {
  my_data <- read_data(model = mymodel, precovid = FALSE,
                       subsidy = subsidy)
  nr_exogenousM <- ifelse(is.null(my_data$zmatM), 0, ncol(my_data$zmatM))
  nr_exogenousR <- ifelse(is.null(my_data$zmatR), 0, ncol(my_data$zmatR))
  set_globals(mymodel)
  my_model_name <- paste0(
    mymodel, ".precov.", 
    paste0(stri_sub(subsidy, from = 1, length = 1), collapse = "")
  )
  mle_out <- readRDS(paste0(
    "output ", out_timing, "/mle_out", my_model_name, ".Rdat"
  ))

  ptm <- proc.time()
  fcasts <- forecast_credit(mle_out$estimate, my_data, which_times = 286 - (45:1), 
                            horizon = 12, scenarios = nsim, 
                            fquantiles = c(0.999,0.5,0.001), 
                            return_all_sim = FALSE, print_life_sign = 1000,
                            stress_scenario = stress_scenario)
  saveRDS(fcasts, paste0("forecast ", timing_name, "/fore", 
                         c(".stress", "")[1 + is.null(stress_scenario)],
                         my_model_name, ".Rdat"))
  print(proc.time() - ptm)
  par(mfrow = c(1,2))
  i <- 2
  j <- 3
  
  a <- fcasts$forecast_bands_marg
  b <- fcasts$realization_marg
  cc <- fcasts$forecast_bands_marg_p
  lims <- range(b[ , i, j]); lims <- sum(lims)/2 + as.double(1.5 * abs(lims %*% c(-1,1)) / 2) * c(-1,1)
  plot(b[ , 2, 3], type = "l", col = "black", lwd = 3, ylim = lims, main = paste("M", mymodel, ": 1m NP def rate", sep = ""))
  for (i1 in sequence(dim(a)[2])) lines(a[, i1, 2, 3], type = "l", col = "blue", lwd = 2, lty = c(3,1,3)[i1])
  
  a <- fcasts$forecast_bands_cum
  b <- fcasts$realization_cum
  cc <- fcasts$forecast_bands_cum_p
  lims <- range(b[ , i, j]); lims <- sum(lims)/2 + as.double(1.5 * abs(lims %*% c(-1,1)) / 2) * c(-1,1)
  plot(b[ , 2, 3], type = "l", xlab = "", col = "black", lwd = 3, ylim = lims, 
       main = paste0("M", mymodel, ": 12m cum NP-D rate",
                     c("stressed", "")[1 + is.null(stress_scenario)])
  )
  for (i1 in sequence(dim(a)[2])) lines(a[, i1, 2, 3], type = "l", col = "blue", lwd = 2, lty = c(3,1,3)[i1])
  for (i1 in sequence(dim(cc)[2])) lines(cc[, i1, 2, 3], type = "l", col = "red", lwd = 1, lty = c(3,1,3)[i1])
}



