# load libraries
library("readxl")
library("seasonal")


# create function
interpolate_IP <- function(Qseries, IP_MSeries) {
  idx <- which(!is.na(Qseries))
  lenidx <- length(idx)
  lenQ <- length(Qseries)
  output <- rep(NA, lenQ)
  for (i1 in 0:2) {
    idx2 <- idx + i1
    idx2 <- idx2[ which(idx2 <= lenQ) ]
    output[idx2] <- Qseries[idx[1:length(idx2)]] / 3 / IP_MSeries[idx2 - 12]
  }
  # fill the tail with last available observation
  aid <- output[lenQ - (50:0)]
  aidsub <- aid[ which(!is.na(aid)) ]
  aid[ which(is.na(aid)) ] <- aidsub[length(aidsub)]
  output[lenQ - (50:0)] <- aid
  return(output)
}

# read in the raw data pull
datafolder <- "../../data/new/"
macrodata <- read_xlsx(paste0(datafolder, "macrodataJBF_raw.xlsx"), range = "A11:H1245")
n <- nrow(macrodata)

# construct annual IP growth
macrodata$IPgrowth <- NA
idx <- sequence(n-12, from = 13)
macrodata$IPgrowth[idx] <- 100 * (macrodata$INDPRO[idx] / macrodata$INDPRO[idx-12] - 1)

# construct Subs in bilions / IP in bilions
# make percentage
macrodata$SubsIP <- 100 * interpolate_IP(macrodata$B096RC1Q027SBEA, macrodata$GVIPT50002S)

# construct TARP in milions / IP in bilions
# make percentage
macrodata$TarpIP <- 100 * interpolate_IP(macrodata$FGCBGSQ027S / 1000, macrodata$GVIPT50002S)

# deseasonalize monthly federal surplus in milions over IP in bilions
# make percentage
yorg <- 100 * (macrodata$MTSDS133FMS / 1000) / macrodata$GVIPT50002S
yorg.ts <- ts(yorg, start=c(1919,1), frequency=12)
# deseasonalize
yorg.sa <- seas(yorg.ts)
# store seasonally adjusted data
macrodata$FedSurplusIP <- NA
macrodata$FedSurplusIP[ !is.na(yorg.sa$x) ] <- final(yorg.sa)
ts.plot(yorg.sa$x,main="Package seasonal")
lines(yorg.sa$data[,1],col="red")

# construct AIG in bilions / IP in bilions
# make percentage
macrodata$AigIP <- 100 * interpolate_IP(macrodata$WAIG, macrodata$GVIPT50002S)


# save the processed data
write.csv(macrodata, file = paste0(datafolder, "macroJBFpreprocessed.csv"))


# read in the raw data pull
macrodata <- read_xlsx(paste0(datafolder, "high_yield_raw.xlsx"), range = "A11:D6613")
macrodata$observation_date <- as.yearmon(macrodata$observation_date)
macrodata <- subset(macrodata, (observation_date >= as.yearmon("1998-01-01")) &
                      (observation_date < as.yearmon("2021-12-01")))
highyields <- data.frame(date = unique(as.yearmon(macrodata$observation_date)))
highyields$HighYield <- tapply(macrodata$BAMLH0A0HYM2, macrodata$observation_date, mean, na.rm = TRUE)
highyields$HighYieldBB <- tapply(macrodata$BAMLH0A1HYBB, macrodata$observation_date, mean, na.rm = TRUE)
highyields$HighYieldCCC <- tapply(macrodata$BAMLH0A3HYC, macrodata$observation_date, mean, na.rm = TRUE)
write.csv(highyields, file = paste0(datafolder, "high_yields_preprocessed.csv"))


